@@ -1,6 +1,6 @@
 fRegress.character <- function(y, data=NULL, betalist=NULL,
                                wt=NULL, y2cMap=NULL, SigmaE=NULL,
-                               method=c('fRegress', 'model'),
+                               method='fRegress',
                                sep='.', ...) {
   fRegress.formula(y=y, data=data, betalist=betalist,
                    wt=wt, y2cMap=y2cMap, SigmaE=SigmaE,
@@ -9,14 +9,47 @@ fRegress.character <- function(y, data=NULL, betalist=NULL,
 
 fRegress.formula <- function(y, data=NULL, betalist=NULL,
                              wt=NULL, y2cMap=NULL, SigmaE=NULL,
-                             method=c('fRegress', 'model'),
-                             sep='.', ...) {
+                             method='fRegress', sep='.', ...) {
   
-  #  Last modified 1 November 2020 by Jim Ramsay
+  #  FREGRESS.FORMULA  Fits a scalar dependent variable using the concurrent
+  #                    functional regression model using inner products
+  #                    of functional covariates and functional regression
+  #                    functions.
+  #
+  #  Arguments:
+  #  Y        ... A numeric vector or a functional data object that is the 
+  #               dependent variable..
+  #  DATA     ... a list object of length p with each list
+  #               containing an object for an independent variable.
+  #               the object may be:
+  #                   a functional data object or
+  #                   a vector
+  #               if XFDLIST is a functional data object or a vector,
+  #               it is converted to a list of length 1.
+  #  BETALIST ... a list object of length p with each list
+  #               containing a functional parameter object for
+  #               the corresponding regression function.  If any of
+  #               these objects is a functional data object, it is
+  #               converted to the default functional parameter object.
+  #               if BETALIST is a functional parameter object
+  #               it is converted to a list of length 1.
+  #  WT       ... a vector of nonnegative weights for observations
+  #  Y2CMAP   ... the matrix mapping from the vector of observed values
+  #               to the coefficients for the dependent variable.
+  #               This is output by function SMOOTH_BASIS.  If this is
+  #               supplied, confidence limits are computed, otherwise not.
+  #  SIGMAE   ... Estimate of the covariances among the residuals.  This
+  #               can only be estimated after a preliminary analysis
+  #. METHOD   ... Type of object returned:  
+  #                 if "fRegress" results of analysis are returned
+  #                 if "model" an argument list fRegressList is returned
+  
+  #  Last modified 29 January 2024 by Jim Ramsay
   
   ##
   ## 1.  get y = left hand side of the formula
   ##
+  
   Formula <- y                    #  character vector
   yName   <- Formula[[2]]         #  name of dependent variable object
   yNm     <- as.character(yName)  #  character strong for name
@@ -25,42 +58,40 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
     stop('The left hand side of formula must be a simple object; ',
          ' instead, LHS = ', as.character(Formula)[2],
          ', which has class = ', class(yName))
-  #
   
   ##
   ## 2.  check the data argument
   ##
   
   dataNames <- names(data)
-  #  extract name of dependent variable
-  y <- {
-    if (yNm %in% dataNames) data[[yNm]]
-    else get(yNm)  # Search by name for object yNm
+  #  extract dependent variable 
+  yvec <- {
+    if (yNm %in% dataNames) data[[yNm]] else get(yNm)  
   }
   #  get the range of the  dependent variable  and 
-  #  obtain the dimensions of the coefficient matrix if y is of class 'fd'
-  #  obtain the dimensions of the y if y is of class 'numeric'
+  #  obtain the dimensions of the coefficient matrix if yvec is of class 'fd'
+  #  obtain the dimensions of the yvec if yvec is of class 'numeric'
   trng <- NULL
   {
-    if(inherits(y, 'fd')){
-      ydim <- dim(y$coefs)
+    if(inherits(yvec, 'fd')){
+      ydim <- dim(yvec$coefs)
       if(is.null(ydim) || (length(ydim)<2)) {
-        y$coefs <- as.matrix(y$coefs)
-        ydim   <- dim(y$coefs)
+        yvec$coefs <- as.matrix(yvec$coefs)
+        ydim   <- dim(yvec$coefs)
       }
       ny   <- ydim[2]
-      trng <- y$basis$rangeval
+      trng <- yvec$basis$rangeval
     } else{
-      if(inherits(y, 'numeric')){
-        ydim <- dim(y)
+      if(inherits(yvec, 'numeric')){
+        ydim <- dim(yvec)
         if(is.null(ydim))
-          ny <- length(y)
+          ny <- length(yvec)
         else
           ny <- ydim[1]
       }
       else
         stop('The left hand side of formula must have class ',
-             'numeric or fd;  instead is ', class(y))
+             'numeric or fd;  instead is ', class(yvec))
     }
   }
   
@@ -72,18 +103,17 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
   xNms     <- allVars[allVars != yNm]
   Terms    <- terms(Formula)
   termLbls <- attr(Terms, 'term.labels')
-  oops     <- which(!(termLbls %in% xNms))
-  if(length(oops) > 0)
-    stop('formula contains terms that fRegress can not handle; ',
-         ' the first one = ', termLbls[oops[1]])
+  oops     <- length(which(!(termLbls %in% xNms))) > 0
+  if(oops) stop('formula contains terms that fRegress can not handle; ',
+                ' the first one = ', termLbls[[1]])
   #
   k1 <- length(allVars)
   type <- rep(NA,k1)
   names(type) <- allVars
   nbasis      <- type
-  if(inherits(y, 'fd')){
-    type[1] <- y$basis$type
-    nb      <- y$basis$nbasis
+  if(inherits(yvec, 'fd')){
+    type[1] <- yvec$basis$type
+    nb      <- yvec$basis$nbasis
     if(!is.null(nb)) nbasis[1] <- nb
   }
   
@@ -91,8 +121,8 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
   ## 4.  Inventory the right hand side
   ##
   
-  k0       <- length(xNms)
-  xfdlist0 <- vector('list', k0)
+  k0              <- length(xNms)
+  xfdlist0        <- vector('list', k0)
   names(xfdlist0) <- xNms
   xNames          <- xfdlist0
   nVars           <- rep(NA, k0)
@@ -101,8 +131,7 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
   for(i in 1:k0) {
     xNm <- xNms[i]
     xi <- {
-      if(xNm %in% dataNames) data[[xNm]]
-      else get(xNm)
+      if(xNm %in% dataNames) data[[xNm]] else get(xNm)
     }
     {
       if(class(xi) %in% c('fd', 'fdPar')) {
@@ -225,11 +254,11 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
     if(nxi != ny){
       cat('ERROR:  variable', xNm, 'has only',
           nxi, 'observations !=', ny,
-          '= the number of observations of y.')
+          '= the number of observations of yvec.')
       oops <- TRUE
     }
   }
-  if(oops)stop('illegal variable on the right hand side.')
+  if (oops) stop('illegal variable on the right hand side.')
   # If no functions found:
   if(is.null(trng)){
     warning("No functions found;  setting rangeval to 0:1")
@@ -255,11 +284,10 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
     xi  <- xfdlist0[[ix]]
     {
       if(inherits(xi, 'fd')) {
-        if(nVars[ix]<2) {
+        if(nVars[ix] < 2) {
           i1            <- i0
           xfdlist[[i0]] <- xi
-        }
-        else {
+        } else {
           #          i1 <- (i1+nVars[ix])
           for(i in 1:nVars[ix]){
             i1  <- i1+1
@@ -274,8 +302,7 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
           if(nVars[ix]<2) {
             i1 <- i0
             xfdlist[[i0]] <- xi
-          }
-          else{
+          } else{
             for(i in 1:nVars[ix]) {
               i1 <- i1+1
               xfdlist[[i1]] <- xi[, i]
@@ -297,49 +324,73 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
         stop('length(betalist) = ', length(betalist),
              ';  must be ', k, ' to match length(xfdlist).')
       betaclass <- sapply(betalist, class)
-      oops      <- which(betaclass != 'fdPar')
-      if(length(oops)>0)
+      oops      <- length(which(betaclass != 'fdPar')) > 0
+      if(oops)
         stop('If betalist is a list, all components must have class ',
              'fdPar;  component ', oops[1], ' has class ',
              betaclass[oops[1]])
-    }
-    else {
+    } else {
       # betalist must be set up
       betalist <- vector('list', k)
       names(betalist) <- xNames.
       for(i in 1:k) {
         if(is.numeric(xfdlist[[i]])) {
-          #  if xfdlist[[i]] is numeric, basis is set up using that  of dependent variable y
-          if(is.numeric(y)) {
+          #. ------------------------------------------------------------------
+          #  if xfdlist[[i]] is numeric, use a constant basis
+          #. ------------------------------------------------------------------
+          if(is.numeric(yvec)) {
+            #. use constant basis
             bbasis        <- create.constant.basis(c(0,1))
-            bfd           <- fd(basisobj=bbasis)
+            bfd           <- fd(1, bbasis)
             betalist[[i]] <- fdPar(bfd)
           } else {
-            #  if 'fd' use the basis of dependent variable
-            if(inherits(y, 'fd')) {
-              bfd           <- with(y, fd(basisobj=basis, fdnames=fdnames))
+            #. basis is set up using that of dependent variable y
+            if(inherits(yvec, 'fd')) {
+              #  if 'fd' use the basis of dependent variable
+              bbasis  <- yvec$basis
+              nbbasis <- bbasis$nbasis
+              bfd     <- with(yvec, fd(matrix(0,nbbasis,1), 
+                                          bbasis, 
+                                          fdnames=fdnames))
               betalist[[i]] <- fdPar(bfd)
-            }
-            else {
-              bfd           <- with(y, fd(basisobj=basis, fdnames=fdnames))
-              betalist[[i]] <- with(y, fdPar(bfd, Lfd, lambda, estimate, penmat))
+            } else {
+              #  if 'fdPar' use the basis of dependent variable$fd
+              bbasis  <- yvec$fd$basis
+              nbbasis <- bbasis$nbasis
+              bfd           <- with(yvec, fd(matrix(0,nbbasis,1), 
+                                                bbasis, 
+                                                fdnames=fdnames))
+              betalist[[i]] <- with(yvec, fdPar(bfd, Lfd, lambda, 
+                                                estimate, penmat))
             }
           }
-        }
-        else {
-          #  use basis for the independent variable
+        } else {
+          #. ------------------------------------------------------------------
+          #   if xfdlist[[i]] is fd or fdPar, 
+          #.  use basis for the independent variable
+          #. ------------------------------------------------------------------
           xfdi <- {
-            if(i>1) xfdlist0[[xL.L0[i-1]]]
-            else    xfdlist[[1]]
+            if(i > 1) xfdlist0[[xL.L0[i-1]]] else xfdlist[[1]]
           }
-          if(inherits(xfdi, 'fd')){
-            bfd           <- with(xfdi, fd(basisobj=basis, fdnames=fdnames))
+          if(inherits(xfdi, 'fd')) {
+            #. xfdi is a functional data object
+            bbasis <- xfdi$basis
+            nbasis <- bbasis$nbasis
+            coef   <- matrix(0,nbasis,1)
+            bfd    <- with(xfdi, fd(coef, bbasis, fdnames=fdnames))
             betalist[[i]] <- fdPar(bfd)
-          }
-          else{
-            bfd           <- with(xfdi$fd, fd(basisobj=basis, fdnames=fdnames))
-            betalist[[i]] <- with(xfdi, fdPar(bfd, Lfd, lambda,
-                                              estimate, penmat))
+          } else {
+            if(inherits(xfdi, 'fdPar')) {
+              #. xfdi is a fdPar object
+              bbasis <- xfdi$fd$basis
+              nbasis <- bbasis$nbasis
+              coef   <- matrix(0,nbasis,1)
+              bfd    <- with(xfdi$fd, fd(coef, bbasis, fdnames=fdnames))
+              betalist[[i]] <- with(xfdi, fdPar(bfd, Lfd, lambda,
+                                                estimate, penmat))
+            } else {
+              stop("Object xfdi is neither fd or fdPar")
+            }
           }
         }
       }
@@ -355,28 +406,26 @@ fRegress.formula <- function(y, data=NULL, betalist=NULL,
       wt <- rep(1, ny)
     else {
       if(length(wt) != ny)
-        stop('length(wt) must match y;  length(wt) = ',
-             length(wt), ' != number of y observations = ', ny)
+        stop('length(wt) must match yvec;  length(wt) = ',
+             length(wt), ' != number of yvec observations = ', ny)
       if(any(wt<0))
         stop('Negative weights found;  not allowed.')
     }
   }
   xiEnd   <- cumsum(nVars)
   xiStart <- c(1, xiEnd[-1])
-  fRegressList <- list(y=y, xfdlist=xfdlist, betalist=betalist, wt=wt)
+  fRegressList <- list(y=yvec, xfdlist=xfdlist, betalist=betalist, wt=wt)
   
   ##
   ## 8.  either output argument list for fRegress() or invoke itcs
   ##
   
-  method <- match.arg(method)
-  if(method=='model') {
+  if(method != "fRegress" && method != "model") 
+    stop("Argument is neither 'fRegress' nor 'model'")
+  if(method == "model") {
     return(fRegressList)
   } else {
-    if(inherits(y, 'fd')) {
-      do.call('fRegress.fd',    fRegressList)
-    } else {
-      do.call('fRegress.double', fRegressList)
-    }
+    if(inherits(yvec, 'fd')) do.call('fRegress.fd',     fRegressList)
+    else                     do.call('fRegress.double', fRegressList)
   }
 }
